// $ANTLR 2.7.4: "ACIItem.g" -> "AntlrACIItemParser.java"$

/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *  
 *    http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License. 
 *  
 */


package org.apache.directory.shared.ldap.aci;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Enumeration;

import org.apache.directory.shared.ldap.filter.AbstractExprNode;
import org.apache.directory.shared.ldap.filter.BranchNode;
import org.apache.directory.shared.ldap.filter.ExprNode;
import org.apache.directory.shared.ldap.filter.FilterParserImpl;
import org.apache.directory.shared.ldap.filter.LeafNode;
import org.apache.directory.shared.ldap.filter.SimpleNode;
import org.apache.directory.shared.ldap.name.NameComponentNormalizer;
import org.apache.directory.shared.ldap.subtree.SubtreeSpecification;
import org.apache.directory.shared.ldap.subtree.SubtreeSpecificationModifier;
import org.apache.directory.shared.ldap.util.ComponentsMonitor;
import org.apache.directory.shared.ldap.util.MandatoryAndOptionalComponentsMonitor;
import org.apache.directory.shared.ldap.util.MandatoryComponentsMonitor;
import org.apache.directory.shared.ldap.util.NamespaceTools;
import org.apache.directory.shared.ldap.util.NoDuplicateKeysMap;
import org.apache.directory.shared.ldap.util.OptionalComponentsMonitor;
import org.apache.directory.shared.ldap.name.LdapDN;
import org.apache.directory.shared.ldap.message.LockableAttributeImpl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;

/**
 * The antlr generated ACIItem parser.
 *
 * @author <a href="mailto:dev@directory.apache.org">Apache Directory Project</a>
 * @version $Rev$
 */
public class AntlrACIItemParser extends antlr.LLkParser       implements AntlrACIItemParserTokenTypes
 {

    private static final Logger log = LoggerFactory.getLogger( AntlrACIItemParser.class );
    
    // subordinate parser instances
    private final FilterParserImpl filterParser = new FilterParserImpl();
    
    private boolean isNormalizing = false;
    NameComponentNormalizer normalizer;
    
    // nonshared global data needed to avoid extensive pass/return stuff
    // these are only used by three first order components
    private String m_identificationTag;
    private AuthenticationLevel m_authenticationLevel;
    private int m_aciPrecedence;
    
    private boolean isItemFirstACIItem;
    
    // shared global data needed to avoid extensive pass/return stuff
    private Set m_protectedItems;
    private Map m_protectedItemsMap;
    private Set m_userClasses;
    private Map m_userClassesMap;
    private Set m_itemPermissions;
    private int m_precedence;
    private Set m_grantsAndDenials;
    private Set m_userPermissions;
    private Map oidsMap;
    
    private Set chopBeforeExclusions;
    private Set chopAfterExclusions;
    private SubtreeSpecificationModifier ssModifier = null;
    
    private ComponentsMonitor mainACIItemComponentsMonitor;
    private ComponentsMonitor itemPermissionComponentsMonitor;
    private ComponentsMonitor userPermissionComponentsMonitor;
    private ComponentsMonitor subtreeSpecificationComponentsMonitor;
    
    
    /**
     * Creates a (normalizing) subordinate DnParser for parsing Names.
     * This method MUST be called for each instance while we cannot do
     * constructor overloading for this class.
     *
     * @return the DnParser to be used for parsing Names
     */
    public void init( Map oidsMap )
    {
    	this.oidsMap = oidsMap;
    }

    /**
     * Sets the NameComponentNormalizer for this parser's dnParser.
     */
    public void setNormalizer(NameComponentNormalizer normalizer)
    {
        this.normalizer = normalizer;
        this.isNormalizing = true;
    }

    private int token2Integer( Token token ) throws RecognitionException
    {
        int i = 0;
        
        try
        {
            i = Integer.parseInt( token.getText());
        }
        catch ( NumberFormatException e )
        {
            throw new RecognitionException( "Value of INTEGER token " +
                                            token.getText() +
                                            " cannot be converted to an Integer" );
        }
        
        return i;
    }

protected AntlrACIItemParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public AntlrACIItemParser(TokenBuffer tokenBuf) {
  this(tokenBuf,1);
}

protected AntlrACIItemParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public AntlrACIItemParser(TokenStream lexer) {
  this(lexer,1);
}

public AntlrACIItemParser(ParserSharedInputState state) {
  super(state,1);
  tokenNames = _tokenNames;
}

	public final  ACIItem  wrapperEntryPoint() throws RecognitionException, TokenStreamException {
		 ACIItem l_ACIItem ;
		
		
		log.debug( "entered wrapperEntryPoint()" );
		l_ACIItem = null;
		
		
		{
		_loop221:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop221;
			}
			
		} while (true);
		}
		l_ACIItem=theACIItem();
		{
		_loop223:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop223;
			}
			
		} while (true);
		}
		match(Token.EOF_TYPE);
		return l_ACIItem ;
	}
	
	public final  ACIItem  theACIItem() throws RecognitionException, TokenStreamException {
		 ACIItem l_ACIItem ;
		
		
		log.debug( "entered theACIItem()" );
		l_ACIItem = null;
		mainACIItemComponentsMonitor = new MandatoryComponentsMonitor( 
		new String [] { "identificationTag", "precedence", "authenticationLevel", "itemOrUserFirst" } );
		
		
		match(OPEN_CURLY);
		{
		_loop226:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop226;
			}
			
		} while (true);
		}
		mainACIItemComponent();
		{
		_loop228:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop228;
			}
			
		} while (true);
		}
		{
		_loop234:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop231:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop231;
					}
					
				} while (true);
				}
				mainACIItemComponent();
				{
				_loop233:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop233;
					}
					
				} while (true);
				}
			}
			else {
				break _loop234;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		if ( !mainACIItemComponentsMonitor.finalStateValid() )
		{
		throw new RecognitionException( "Missing mandatory ACIItem components: " 
		+ mainACIItemComponentsMonitor.getRemainingComponents() );
		}
		
		if ( isItemFirstACIItem )
		{
		l_ACIItem = new ItemFirstACIItem(
		m_identificationTag,
		m_precedence,
		m_authenticationLevel,
		m_protectedItems,
		m_itemPermissions );
		}
		else
		{
		l_ACIItem = new UserFirstACIItem(
		m_identificationTag,
		m_aciPrecedence,
		m_authenticationLevel,
		m_userClasses,
		m_userPermissions );
		}
		
		
		return l_ACIItem ;
	}
	
	public final void mainACIItemComponent() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered mainACIItemComponent()" );
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ID_identificationTag:
			{
				aci_identificationTag();
				
				mainACIItemComponentsMonitor.useComponent( "identificationTag" );
				
				break;
			}
			case ID_precedence:
			{
				aci_precedence();
				
				mainACIItemComponentsMonitor.useComponent( "precedence" );
				
				break;
			}
			case ID_authenticationLevel:
			{
				aci_authenticationLevel();
				
				mainACIItemComponentsMonitor.useComponent( "authenticationLevel" );
				
				break;
			}
			case ID_itemOrUserFirst:
			{
				aci_itemOrUserFirst();
				
				mainACIItemComponentsMonitor.useComponent( "itemOrUserFirst" );
				
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( e.getMessage() );
			
		}
	}
	
	public final void aci_identificationTag() throws RecognitionException, TokenStreamException {
		
		Token  token = null;
		
		log.debug( "entered aci_identificationTag()" );
		
		
		match(ID_identificationTag);
		{
		int _cnt238=0;
		_loop238:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt238>=1 ) { break _loop238; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt238++;
		} while (true);
		}
		token = LT(1);
		match(SAFEUTF8STRING);
		
		m_identificationTag = token.getText();
		
	}
	
	public final void aci_precedence() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered aci_precedence()" );
		
		
		precedence();
		
		m_aciPrecedence = m_precedence;
		
	}
	
	public final void aci_authenticationLevel() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered aci_authenticationLevel()" );
		
		
		match(ID_authenticationLevel);
		{
		int _cnt245=0;
		_loop245:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt245>=1 ) { break _loop245; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt245++;
		} while (true);
		}
		authenticationLevel();
	}
	
	public final void aci_itemOrUserFirst() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered aci_itemOrUserFirst()" );
		
		
		match(ID_itemOrUserFirst);
		{
		int _cnt249=0;
		_loop249:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt249>=1 ) { break _loop249; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt249++;
		} while (true);
		}
		itemOrUserFirst();
	}
	
	public final void precedence() throws RecognitionException, TokenStreamException {
		
		Token  token = null;
		
		log.debug( "entered precedence()" );
		
		
		match(ID_precedence);
		{
		int _cnt242=0;
		_loop242:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt242>=1 ) { break _loop242; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt242++;
		} while (true);
		}
		token = LT(1);
		match(INTEGER);
		
		m_precedence = token2Integer( token );
		if( m_precedence < 0 || m_precedence > 255 )
		{
		throw new RecognitionException( "Expecting INTEGER token having an Integer value between 0 and 255, found " + m_precedence );
		}
		
	}
	
	public final void authenticationLevel() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered authenticationLevel()" );
		
		
		switch ( LA(1)) {
		case ID_none:
		{
			match(ID_none);
			
			m_authenticationLevel = AuthenticationLevel.NONE;
			
			break;
		}
		case ID_simple:
		{
			match(ID_simple);
			
			m_authenticationLevel = AuthenticationLevel.SIMPLE;
			
			break;
		}
		case ID_strong:
		{
			match(ID_strong);
			
			m_authenticationLevel = AuthenticationLevel.STRONG;
			
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void itemOrUserFirst() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered itemOrUserFirst()" );
		
		
		switch ( LA(1)) {
		case ID_itemFirst:
		{
			itemFirst();
			break;
		}
		case ID_userFirst:
		{
			userFirst();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void itemFirst() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered itemFirst()" );
		
		
		match(ID_itemFirst);
		{
		_loop253:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop253;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop255:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop255;
			}
			
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop257:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop257;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_protectedItems:
		{
			protectedItems();
			{
			_loop260:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop260;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop262:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop262;
				}
				
			} while (true);
			}
			itemPermissions();
			break;
		}
		case ID_itemPermissions:
		{
			itemPermissions();
			{
			_loop264:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop264;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop266:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop266;
				}
				
			} while (true);
			}
			protectedItems();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop268:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop268;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		isItemFirstACIItem = true;
		
	}
	
	public final void userFirst() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered userFirst()" );
		
		
		match(ID_userFirst);
		{
		_loop271:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop271;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop273:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop273;
			}
			
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop275:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop275;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_userClasses:
		{
			userClasses();
			{
			_loop278:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop278;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop280:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop280;
				}
				
			} while (true);
			}
			userPermissions();
			break;
		}
		case ID_userPermissions:
		{
			userPermissions();
			{
			_loop282:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop282;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop284:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop284;
				}
				
			} while (true);
			}
			userClasses();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop286:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop286;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		isItemFirstACIItem = false;
		
	}
	
	public final void protectedItems() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered protectedItems()" );
		m_protectedItemsMap = new NoDuplicateKeysMap();
		
		
		try {      // for error handling
			match(ID_protectedItems);
			{
			_loop289:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop289;
				}
				
			} while (true);
			}
			match(OPEN_CURLY);
			{
			_loop291:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop291;
				}
				
			} while (true);
			}
			{
			switch ( LA(1)) {
			case ATTRIBUTE_VALUE_CANDIDATE:
			case RANGE_OF_VALUES_CANDIDATE:
			case ID_entry:
			case ID_allUserAttributeTypes:
			case ID_attributeType:
			case ID_allAttributeValues:
			case ID_allUserAttributeTypesAndValues:
			case ID_selfValue:
			case ID_maxValueCount:
			case ID_maxImmSub:
			case ID_restrictedBy:
			case ID_classes:
			{
				protectedItem();
				{
				_loop294:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop294;
					}
					
				} while (true);
				}
				{
				_loop300:
				do {
					if ((LA(1)==SEP)) {
						match(SEP);
						{
						_loop297:
						do {
							if ((LA(1)==SP)) {
								match(SP);
							}
							else {
								break _loop297;
							}
							
						} while (true);
						}
						protectedItem();
						{
						_loop299:
						do {
							if ((LA(1)==SP)) {
								match(SP);
							}
							else {
								break _loop299;
							}
							
						} while (true);
						}
					}
					else {
						break _loop300;
					}
					
				} while (true);
				}
				break;
			}
			case CLOSE_CURLY:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(CLOSE_CURLY);
			
			m_protectedItems = new HashSet( m_protectedItemsMap.values() );
			
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( "Protected Items cannot be duplicated. " + e.getMessage() );
			
		}
	}
	
	public final void itemPermissions() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered itemPermissions()" );
		m_itemPermissions = new HashSet();
		ItemPermission l_itemPermission = null;
		
		
		match(ID_itemPermissions);
		{
		int _cnt405=0;
		_loop405:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt405>=1 ) { break _loop405; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt405++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop407:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop407;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case OPEN_CURLY:
		{
			l_itemPermission=itemPermission();
			{
			_loop410:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop410;
				}
				
			} while (true);
			}
			
			m_itemPermissions.add( l_itemPermission );
			
			{
			_loop416:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop413:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop413;
						}
						
					} while (true);
					}
					l_itemPermission=itemPermission();
					{
					_loop415:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop415;
						}
						
					} while (true);
					}
					
					m_itemPermissions.add( l_itemPermission );
					
				}
				else {
					break _loop416;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
	}
	
	public final void userClasses() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered userClasses()" );
		m_userClassesMap = new NoDuplicateKeysMap();
		
		
		try {      // for error handling
			match(ID_userClasses);
			{
			int _cnt446=0;
			_loop446:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt446>=1 ) { break _loop446; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt446++;
			} while (true);
			}
			match(OPEN_CURLY);
			{
			_loop448:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop448;
				}
				
			} while (true);
			}
			{
			switch ( LA(1)) {
			case ID_allUsers:
			case ID_thisEntry:
			case ID_name:
			case ID_userGroup:
			case ID_subtree:
			{
				userClass();
				{
				_loop451:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop451;
					}
					
				} while (true);
				}
				{
				_loop457:
				do {
					if ((LA(1)==SEP)) {
						match(SEP);
						{
						_loop454:
						do {
							if ((LA(1)==SP)) {
								match(SP);
							}
							else {
								break _loop454;
							}
							
						} while (true);
						}
						userClass();
						{
						_loop456:
						do {
							if ((LA(1)==SP)) {
								match(SP);
							}
							else {
								break _loop456;
							}
							
						} while (true);
						}
					}
					else {
						break _loop457;
					}
					
				} while (true);
				}
				break;
			}
			case CLOSE_CURLY:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(CLOSE_CURLY);
			
			m_userClasses  = new HashSet( m_userClassesMap.values() );
			
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( "User Classes cannot be duplicated. " + e.getMessage() );
			
		}
	}
	
	public final void userPermissions() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered userPermissions()" );
		m_userPermissions = new HashSet();
		UserPermission l_userPermission = null;
		
		
		match(ID_userPermissions);
		{
		int _cnt502=0;
		_loop502:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt502>=1 ) { break _loop502; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt502++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop504:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop504;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case OPEN_CURLY:
		{
			l_userPermission=userPermission();
			{
			_loop507:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop507;
				}
				
			} while (true);
			}
			
			m_userPermissions.add( l_userPermission );
			
			{
			_loop513:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop510:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop510;
						}
						
					} while (true);
					}
					l_userPermission=userPermission();
					{
					_loop512:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop512;
						}
						
					} while (true);
					}
					
					m_userPermissions.add( l_userPermission );
					
				}
				else {
					break _loop513;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
	}
	
	public final void protectedItem() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered protectedItem()" );
		
		
		switch ( LA(1)) {
		case ID_entry:
		{
			entry();
			break;
		}
		case ID_allUserAttributeTypes:
		{
			allUserAttributeTypes();
			break;
		}
		case ID_attributeType:
		{
			attributeType();
			break;
		}
		case ID_allAttributeValues:
		{
			allAttributeValues();
			break;
		}
		case ID_allUserAttributeTypesAndValues:
		{
			allUserAttributeTypesAndValues();
			break;
		}
		case ATTRIBUTE_VALUE_CANDIDATE:
		{
			attributeValue();
			break;
		}
		case ID_selfValue:
		{
			selfValue();
			break;
		}
		case RANGE_OF_VALUES_CANDIDATE:
		{
			rangeOfValues();
			break;
		}
		case ID_maxValueCount:
		{
			maxValueCount();
			break;
		}
		case ID_maxImmSub:
		{
			maxImmSub();
			break;
		}
		case ID_restrictedBy:
		{
			restrictedBy();
			break;
		}
		case ID_classes:
		{
			classes();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void entry() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered entry()" );  
		
		
		match(ID_entry);
		
		m_protectedItemsMap.put( "entry", ProtectedItem.ENTRY );
		
	}
	
	public final void allUserAttributeTypes() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered allUserAttributeTypes()" );
		
		
		match(ID_allUserAttributeTypes);
		
		m_protectedItemsMap.put( "allUserAttributeTypes", ProtectedItem.ALL_USER_ATTRIBUTE_TYPES );
		
	}
	
	public final void attributeType() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered attributeType()" );
		Set l_attributeTypeSet = null;
		
		
		match(ID_attributeType);
		{
		int _cnt306=0;
		_loop306:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt306>=1 ) { break _loop306; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt306++;
		} while (true);
		}
		l_attributeTypeSet=attributeTypeSet();
		
		m_protectedItemsMap.put( "attributeType", new ProtectedItem.AttributeType( l_attributeTypeSet ) );
		
	}
	
	public final void allAttributeValues() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered allAttributeValues()" );
		Set l_attributeTypeSet = null;
		
		
		match(ID_allAttributeValues);
		{
		int _cnt309=0;
		_loop309:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt309>=1 ) { break _loop309; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt309++;
		} while (true);
		}
		l_attributeTypeSet=attributeTypeSet();
		
		m_protectedItemsMap.put( "allAttributeValues", new ProtectedItem.AllAttributeValues( l_attributeTypeSet ) );
		
	}
	
	public final void allUserAttributeTypesAndValues() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered allUserAttributeTypesAndValues()" );
		
		
		match(ID_allUserAttributeTypesAndValues);
		
		m_protectedItemsMap.put( "allUserAttributeTypesAndValues", ProtectedItem.ALL_USER_ATTRIBUTE_TYPES_AND_VALUES );
		
	}
	
	public final void attributeValue() throws RecognitionException, TokenStreamException {
		
		Token  token = null;
		
		log.debug( "entered attributeValue()" );
		String attributeTypeAndValue = null;
		String attributeType = null;
		String attributeValue = null;
		Set attributeSet = new HashSet();
		
		
		try {      // for error handling
			token = LT(1);
			match(ATTRIBUTE_VALUE_CANDIDATE);
			
			// A Dn can be considered as a set of attributeTypeAndValues
			// So, parse the set as a Dn and extract each attributeTypeAndValue
			LdapDN attributeTypeAndValueSetAsDn = new LdapDN( token.getText() );
			attributeTypeAndValueSetAsDn.normalize( oidsMap );
			Enumeration attributeTypeAndValueSet = attributeTypeAndValueSetAsDn.getAll();
			while ( attributeTypeAndValueSet.hasMoreElements() )
			{
			attributeTypeAndValue = ( String ) attributeTypeAndValueSet.nextElement();
			attributeType = NamespaceTools.getRdnAttribute( attributeTypeAndValue );
			attributeValue = NamespaceTools.getRdnValue( attributeTypeAndValue );
			attributeSet.add( new LockableAttributeImpl( attributeType, attributeValue ) );
			log.debug( "An attributeTypeAndValue from the set: " + attributeType + "=" +  attributeValue);
			}
			m_protectedItemsMap.put( "attributeValue", new ProtectedItem.AttributeValue( attributeSet ) );
			
		}
		catch (Exception e) {
			
			throw new RecognitionException( "dnParser failed for " + token.getText() + " , " + e.getMessage() );
			
		}
	}
	
	public final void selfValue() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered selfValue()" );
		Set l_attributeTypeSet = null;
		
		
		match(ID_selfValue);
		{
		int _cnt314=0;
		_loop314:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt314>=1 ) { break _loop314; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt314++;
		} while (true);
		}
		l_attributeTypeSet=attributeTypeSet();
		
		m_protectedItemsMap.put( "sefValue", new ProtectedItem.SelfValue( l_attributeTypeSet ) );
		
	}
	
	public final void rangeOfValues() throws RecognitionException, TokenStreamException {
		
		Token  token = null;
		
		log.debug( "entered rangeOfValues()" );
		
		
		try {      // for error handling
			token = LT(1);
			match(RANGE_OF_VALUES_CANDIDATE);
			
			m_protectedItemsMap.put( "rangeOfValues",
			new ProtectedItem.RangeOfValues(
			filterParser.parse( token.getText() ) ) );
			log.debug( "filterParser parsed " + token.getText() );
			
		}
		catch (Exception e) {
			
			throw new RecognitionException( "filterParser failed. " + e.getMessage() );
			
		}
	}
	
	public final void maxValueCount() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered maxValueCount()" );
		ProtectedItem.MaxValueCountItem l_maxValueCount = null;
		Set maxValueCountSet = new HashSet();
		
		
		match(ID_maxValueCount);
		{
		int _cnt318=0;
		_loop318:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt318>=1 ) { break _loop318; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt318++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop320:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop320;
			}
			
		} while (true);
		}
		l_maxValueCount=aMaxValueCount();
		{
		_loop322:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop322;
			}
			
		} while (true);
		}
		
		maxValueCountSet.add( l_maxValueCount );
		
		{
		_loop328:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop325:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop325;
					}
					
				} while (true);
				}
				l_maxValueCount=aMaxValueCount();
				{
				_loop327:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop327;
					}
					
				} while (true);
				}
				
				maxValueCountSet.add( l_maxValueCount );
				
			}
			else {
				break _loop328;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		m_protectedItemsMap.put( "maxValueCount", new ProtectedItem.MaxValueCount( maxValueCountSet ) );
		
	}
	
	public final void maxImmSub() throws RecognitionException, TokenStreamException {
		
		Token  token = null;
		
		log.debug( "entered maxImmSub()" );
		
		
		match(ID_maxImmSub);
		{
		int _cnt353=0;
		_loop353:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt353>=1 ) { break _loop353; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt353++;
		} while (true);
		}
		token = LT(1);
		match(INTEGER);
		
		
		m_protectedItemsMap.put( "maxImmSub",
		new ProtectedItem.MaxImmSub(
		token2Integer( token ) ) );
		
	}
	
	public final void restrictedBy() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered restrictedBy()" );
		ProtectedItem.RestrictedByItem l_restrictedValue = null;
		Set l_restrictedBy = new HashSet();
		
		
		match(ID_restrictedBy);
		{
		int _cnt356=0;
		_loop356:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt356>=1 ) { break _loop356; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt356++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop358:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop358;
			}
			
		} while (true);
		}
		l_restrictedValue=restrictedValue();
		{
		_loop360:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop360;
			}
			
		} while (true);
		}
		
		l_restrictedBy.add( l_restrictedValue );
		
		{
		_loop366:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop363:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop363;
					}
					
				} while (true);
				}
				l_restrictedValue=restrictedValue();
				{
				_loop365:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop365;
					}
					
				} while (true);
				}
				
				l_restrictedBy.add( l_restrictedValue );
				
			}
			else {
				break _loop366;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		m_protectedItemsMap.put( "restrictedBy", new ProtectedItem.RestrictedBy( l_restrictedBy ) );
		
	}
	
	public final void classes() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered classes()" );
		ExprNode l_classes = null;
		
		
		match(ID_classes);
		{
		int _cnt402=0;
		_loop402:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt402>=1 ) { break _loop402; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt402++;
		} while (true);
		}
		l_classes=refinement();
		
		m_protectedItemsMap.put( "classes", new ProtectedItem.Classes( l_classes ) );
		
	}
	
	public final  Set  attributeTypeSet() throws RecognitionException, TokenStreamException {
		 Set l_attributeTypeSet ;
		
		
		log.debug( "entered attributeTypeSet()" );
		String l_oid = null;
		l_attributeTypeSet = new HashSet();
		
		
		match(OPEN_CURLY);
		{
		_loop391:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop391;
			}
			
		} while (true);
		}
		l_oid=oid();
		{
		_loop393:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop393;
			}
			
		} while (true);
		}
		
		l_attributeTypeSet.add( l_oid );
		
		{
		_loop399:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop396:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop396;
					}
					
				} while (true);
				}
				l_oid=oid();
				{
				_loop398:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop398;
					}
					
				} while (true);
				}
				
				l_attributeTypeSet.add( l_oid );
				
			}
			else {
				break _loop399;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		return l_attributeTypeSet ;
	}
	
	public final  ProtectedItem.MaxValueCountItem  aMaxValueCount() throws RecognitionException, TokenStreamException {
		 ProtectedItem.MaxValueCountItem l_maxValueCount ;
		
		Token  token1 = null;
		Token  token2 = null;
		
		log.debug( "entered aMaxValueCount()" );
		l_maxValueCount = null;
		String l_oid = null;
		Token token = null;
		
		
		match(OPEN_CURLY);
		{
		_loop331:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop331;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_type:
		{
			match(ID_type);
			{
			int _cnt334=0;
			_loop334:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt334>=1 ) { break _loop334; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt334++;
			} while (true);
			}
			l_oid=oid();
			{
			_loop336:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop336;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop338:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop338;
				}
				
			} while (true);
			}
			match(ID_maxCount);
			{
			int _cnt340=0;
			_loop340:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt340>=1 ) { break _loop340; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt340++;
			} while (true);
			}
			token1 = LT(1);
			match(INTEGER);
			token = token1;
			break;
		}
		case ID_maxCount:
		{
			match(ID_maxCount);
			{
			int _cnt342=0;
			_loop342:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt342>=1 ) { break _loop342; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt342++;
			} while (true);
			}
			token2 = LT(1);
			match(INTEGER);
			{
			_loop344:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop344;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop346:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop346;
				}
				
			} while (true);
			}
			match(ID_type);
			{
			int _cnt348=0;
			_loop348:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt348>=1 ) { break _loop348; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt348++;
			} while (true);
			}
			l_oid=oid();
			token = token2;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop350:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop350;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		l_maxValueCount = new ProtectedItem.MaxValueCountItem( l_oid, token2Integer( token ) );
		
		return l_maxValueCount ;
	}
	
	public final  String  oid() throws RecognitionException, TokenStreamException {
		 String result ;
		
		
		log.debug( "entered oid()" );
		result = null;
		Token token = null;
		
		
		token = LT( 1 );
		{
		switch ( LA(1)) {
		case DESCR:
		{
			match(DESCR);
			break;
		}
		case NUMERICOID:
		{
			match(NUMERICOID);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		
		result = token.getText();
		log.debug( "recognized an oid: " + result );
		
		return result ;
	}
	
	public final  ProtectedItem.RestrictedByItem  restrictedValue() throws RecognitionException, TokenStreamException {
		 ProtectedItem.RestrictedByItem l_restrictedValue ;
		
		
		log.debug( "entered restrictedValue()" );
		String typeOid = null;
		String valuesInOid = null;
		l_restrictedValue = null;
		
		
		match(OPEN_CURLY);
		{
		_loop369:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop369;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_type:
		{
			match(ID_type);
			{
			int _cnt372=0;
			_loop372:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt372>=1 ) { break _loop372; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt372++;
			} while (true);
			}
			typeOid=oid();
			{
			_loop374:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop374;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop376:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop376;
				}
				
			} while (true);
			}
			match(ID_valuesIn);
			{
			int _cnt378=0;
			_loop378:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt378>=1 ) { break _loop378; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt378++;
			} while (true);
			}
			valuesInOid=oid();
			break;
		}
		case ID_valuesIn:
		{
			match(ID_valuesIn);
			{
			int _cnt380=0;
			_loop380:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt380>=1 ) { break _loop380; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt380++;
			} while (true);
			}
			valuesInOid=oid();
			{
			_loop382:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop382;
				}
				
			} while (true);
			}
			match(SEP);
			{
			_loop384:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop384;
				}
				
			} while (true);
			}
			match(ID_type);
			{
			int _cnt386=0;
			_loop386:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					if ( _cnt386>=1 ) { break _loop386; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt386++;
			} while (true);
			}
			typeOid=oid();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop388:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop388;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		l_restrictedValue = new ProtectedItem.RestrictedByItem( typeOid, valuesInOid );
		
		return l_restrictedValue ;
	}
	
	public final  ExprNode  refinement() throws RecognitionException, TokenStreamException {
		 ExprNode node ;
		
		
		log.debug( "entered refinement()" );
		node = null;
		
		
		switch ( LA(1)) {
		case ID_item:
		{
			node=item();
			break;
		}
		case ID_and:
		{
			node=and();
			break;
		}
		case ID_or:
		{
			node=or();
			break;
		}
		case ID_not:
		{
			node=not();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return node ;
	}
	
	public final  ItemPermission  itemPermission() throws RecognitionException, TokenStreamException {
		 ItemPermission l_itemPermission ;
		
		
		log.debug( "entered itemPermission()" );
		l_itemPermission = null;
		itemPermissionComponentsMonitor = new MandatoryAndOptionalComponentsMonitor( 
		new String [] { "userClasses", "grantsAndDenials" }, new String [] { "precedence" } );
		
		
		match(OPEN_CURLY);
		{
		_loop419:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop419;
			}
			
		} while (true);
		}
		anyItemPermission();
		{
		_loop421:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop421;
			}
			
		} while (true);
		}
		{
		_loop427:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop424:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop424;
					}
					
				} while (true);
				}
				anyItemPermission();
				{
				_loop426:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop426;
					}
					
				} while (true);
				}
			}
			else {
				break _loop427;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		if ( !itemPermissionComponentsMonitor.finalStateValid() )
		{
		throw new RecognitionException( "Missing mandatory itemPermission components: " 
		+ itemPermissionComponentsMonitor.getRemainingComponents() );
		}
		
		l_itemPermission = new ItemPermission( m_precedence, m_grantsAndDenials, m_userClasses );
		
		return l_itemPermission ;
	}
	
	public final void anyItemPermission() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ID_precedence:
			{
				precedence();
				
				itemPermissionComponentsMonitor.useComponent( "precedence" );
				
				break;
			}
			case ID_userClasses:
			{
				userClasses();
				
				itemPermissionComponentsMonitor.useComponent( "userClasses" );
				
				break;
			}
			case ID_grantsAndDenials:
			{
				grantsAndDenials();
				
				itemPermissionComponentsMonitor.useComponent( "grantsAndDenials" );
				
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( e.getMessage() );
			
		}
	}
	
	public final void grantsAndDenials() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered grantsAndDenials()" );
		m_grantsAndDenials = new HashSet();
		GrantAndDenial l_grantAndDenial = null;
		
		
		match(ID_grantsAndDenials);
		{
		int _cnt431=0;
		_loop431:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt431>=1 ) { break _loop431; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt431++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop433:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop433;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_grantAdd:
		case ID_denyAdd:
		case ID_grantDiscloseOnError:
		case ID_denyDiscloseOnError:
		case ID_grantRead:
		case ID_denyRead:
		case ID_grantRemove:
		case ID_denyRemove:
		case ID_grantBrowse:
		case ID_denyBrowse:
		case ID_grantExport:
		case ID_denyExport:
		case ID_grantImport:
		case ID_denyImport:
		case ID_grantModify:
		case ID_denyModify:
		case ID_grantRename:
		case ID_denyRename:
		case ID_grantReturnDN:
		case ID_denyReturnDN:
		case ID_grantCompare:
		case ID_denyCompare:
		case ID_grantFilterMatch:
		case ID_denyFilterMatch:
		case ID_grantInvoke:
		case ID_denyInvoke:
		{
			l_grantAndDenial=grantAndDenial();
			{
			_loop436:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop436;
				}
				
			} while (true);
			}
			
			if ( !m_grantsAndDenials.add( l_grantAndDenial ))
			{
			throw new RecognitionException( "Duplicated GrantAndDenial bit: " + l_grantAndDenial );
			}
			
			{
			_loop442:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop439:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop439;
						}
						
					} while (true);
					}
					l_grantAndDenial=grantAndDenial();
					{
					_loop441:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop441;
						}
						
					} while (true);
					}
					
					if ( !m_grantsAndDenials.add( l_grantAndDenial ))
					{
					throw new RecognitionException( "Duplicated GrantAndDenial bit: " + l_grantAndDenial );
					}
					
				}
				else {
					break _loop442;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
	}
	
	public final  GrantAndDenial  grantAndDenial() throws RecognitionException, TokenStreamException {
		 GrantAndDenial l_grantAndDenial ;
		
		
		log.debug( "entered grantAndDenialsBit()" );
		l_grantAndDenial = null;
		
		
		switch ( LA(1)) {
		case ID_grantAdd:
		{
			match(ID_grantAdd);
			l_grantAndDenial = GrantAndDenial.GRANT_ADD;
			break;
		}
		case ID_denyAdd:
		{
			match(ID_denyAdd);
			l_grantAndDenial = GrantAndDenial.DENY_ADD;
			break;
		}
		case ID_grantDiscloseOnError:
		{
			match(ID_grantDiscloseOnError);
			l_grantAndDenial = GrantAndDenial.GRANT_DISCLOSE_ON_ERROR;
			break;
		}
		case ID_denyDiscloseOnError:
		{
			match(ID_denyDiscloseOnError);
			l_grantAndDenial = GrantAndDenial.DENY_DISCLOSE_ON_ERROR;
			break;
		}
		case ID_grantRead:
		{
			match(ID_grantRead);
			l_grantAndDenial = GrantAndDenial.GRANT_READ;
			break;
		}
		case ID_denyRead:
		{
			match(ID_denyRead);
			l_grantAndDenial = GrantAndDenial.DENY_READ;
			break;
		}
		case ID_grantRemove:
		{
			match(ID_grantRemove);
			l_grantAndDenial = GrantAndDenial.GRANT_REMOVE;
			break;
		}
		case ID_denyRemove:
		{
			match(ID_denyRemove);
			l_grantAndDenial = GrantAndDenial.DENY_REMOVE;
			break;
		}
		case ID_grantBrowse:
		{
			match(ID_grantBrowse);
			l_grantAndDenial = GrantAndDenial.GRANT_BROWSE;
			break;
		}
		case ID_denyBrowse:
		{
			match(ID_denyBrowse);
			l_grantAndDenial = GrantAndDenial.DENY_BROWSE;
			break;
		}
		case ID_grantExport:
		{
			match(ID_grantExport);
			l_grantAndDenial = GrantAndDenial.GRANT_EXPORT;
			break;
		}
		case ID_denyExport:
		{
			match(ID_denyExport);
			l_grantAndDenial = GrantAndDenial.DENY_EXPORT;
			break;
		}
		case ID_grantImport:
		{
			match(ID_grantImport);
			l_grantAndDenial = GrantAndDenial.GRANT_IMPORT;
			break;
		}
		case ID_denyImport:
		{
			match(ID_denyImport);
			l_grantAndDenial = GrantAndDenial.DENY_IMPORT;
			break;
		}
		case ID_grantModify:
		{
			match(ID_grantModify);
			l_grantAndDenial = GrantAndDenial.GRANT_MODIFY;
			break;
		}
		case ID_denyModify:
		{
			match(ID_denyModify);
			l_grantAndDenial = GrantAndDenial.DENY_MODIFY;
			break;
		}
		case ID_grantRename:
		{
			match(ID_grantRename);
			l_grantAndDenial = GrantAndDenial.GRANT_RENAME;
			break;
		}
		case ID_denyRename:
		{
			match(ID_denyRename);
			l_grantAndDenial = GrantAndDenial.DENY_RENAME;
			break;
		}
		case ID_grantReturnDN:
		{
			match(ID_grantReturnDN);
			l_grantAndDenial = GrantAndDenial.GRANT_RETURN_DN;
			break;
		}
		case ID_denyReturnDN:
		{
			match(ID_denyReturnDN);
			l_grantAndDenial = GrantAndDenial.DENY_RETURN_DN;
			break;
		}
		case ID_grantCompare:
		{
			match(ID_grantCompare);
			l_grantAndDenial = GrantAndDenial.GRANT_COMPARE;
			break;
		}
		case ID_denyCompare:
		{
			match(ID_denyCompare);
			l_grantAndDenial = GrantAndDenial.DENY_COMPARE;
			break;
		}
		case ID_grantFilterMatch:
		{
			match(ID_grantFilterMatch);
			l_grantAndDenial = GrantAndDenial.GRANT_FILTER_MATCH;
			break;
		}
		case ID_denyFilterMatch:
		{
			match(ID_denyFilterMatch);
			l_grantAndDenial = GrantAndDenial.DENY_FILTER_MATCH;
			break;
		}
		case ID_grantInvoke:
		{
			match(ID_grantInvoke);
			l_grantAndDenial = GrantAndDenial.GRANT_INVOKE;
			break;
		}
		case ID_denyInvoke:
		{
			match(ID_denyInvoke);
			l_grantAndDenial = GrantAndDenial.DENY_INVOKE;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return l_grantAndDenial ;
	}
	
	public final void userClass() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered userClasses()" );
		
		
		switch ( LA(1)) {
		case ID_allUsers:
		{
			allUsers();
			break;
		}
		case ID_thisEntry:
		{
			thisEntry();
			break;
		}
		case ID_name:
		{
			name();
			break;
		}
		case ID_userGroup:
		{
			userGroup();
			break;
		}
		case ID_subtree:
		{
			subtree();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void allUsers() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered allUsers()" );
		
		
		match(ID_allUsers);
		
		m_userClassesMap.put( "allUsers", UserClass.ALL_USERS );
		
	}
	
	public final void thisEntry() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered thisEntry()" );
		
		
		match(ID_thisEntry);
		
		m_userClassesMap.put( "thisEntry", UserClass.THIS_ENTRY );
		
	}
	
	public final void name() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered name()" );
		Set l_name = new HashSet();
		LdapDN l_distinguishedName = null;
		
		
		match(ID_name);
		{
		int _cnt463=0;
		_loop463:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt463>=1 ) { break _loop463; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt463++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop465:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop465;
			}
			
		} while (true);
		}
		l_distinguishedName=distinguishedName();
		{
		_loop467:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop467;
			}
			
		} while (true);
		}
		
		l_name.add( l_distinguishedName );
		
		{
		_loop473:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop470:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop470;
					}
					
				} while (true);
				}
				l_distinguishedName=distinguishedName();
				{
				_loop472:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop472;
					}
					
				} while (true);
				}
				
				l_name.add( l_distinguishedName );
				
			}
			else {
				break _loop473;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		m_userClassesMap.put( "name", new UserClass.Name( l_name ) );
		
	}
	
	public final void userGroup() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered userGroup()" );
		Set l_userGroup = new HashSet();
		LdapDN l_distinguishedName = null;
		
		
		match(ID_userGroup);
		{
		int _cnt476=0;
		_loop476:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt476>=1 ) { break _loop476; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt476++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop478:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop478;
			}
			
		} while (true);
		}
		l_distinguishedName=distinguishedName();
		{
		_loop480:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop480;
			}
			
		} while (true);
		}
		
		l_userGroup.add( l_distinguishedName );
		
		{
		_loop486:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop483:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop483;
					}
					
				} while (true);
				}
				l_distinguishedName=distinguishedName();
				{
				_loop485:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop485;
					}
					
				} while (true);
				}
				
				l_userGroup.add( l_distinguishedName );
				
			}
			else {
				break _loop486;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		m_userClassesMap.put( "userGroup", new UserClass.UserGroup( l_userGroup ) );
		
	}
	
	public final void subtree() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered subtree()" );
		Set l_subtree = new HashSet();
		SubtreeSpecification l_subtreeSpecification = null;    
		
		
		match(ID_subtree);
		{
		int _cnt489=0;
		_loop489:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt489>=1 ) { break _loop489; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt489++;
		} while (true);
		}
		match(OPEN_CURLY);
		{
		_loop491:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop491;
			}
			
		} while (true);
		}
		l_subtreeSpecification=subtreeSpecification();
		{
		_loop493:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop493;
			}
			
		} while (true);
		}
		
		l_subtree.add( l_subtreeSpecification );
		
		{
		_loop499:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop496:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop496;
					}
					
				} while (true);
				}
				l_subtreeSpecification=subtreeSpecification();
				{
				_loop498:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop498;
					}
					
				} while (true);
				}
				
				l_subtree.add( l_subtreeSpecification );
				
			}
			else {
				break _loop499;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		m_userClassesMap.put( "subtree", new UserClass.Subtree( l_subtree ) );
		
	}
	
	public final  LdapDN  distinguishedName() throws RecognitionException, TokenStreamException {
		 LdapDN name ;
		
		Token  token = null;
		
		log.debug( "entered distinguishedName()" );
		name = null;
		
		
		try {      // for error handling
			token = LT(1);
			match(SAFEUTF8STRING);
			
			name = new LdapDN( token.getText() );
			name.normalize( oidsMap );
			log.debug( "recognized a DistinguishedName: " + token.getText() );
			
		}
		catch (Exception e) {
			
			throw new RecognitionException( "dnParser failed for " + token.getText() + " " + e.getMessage() );
			
		}
		return name ;
	}
	
	public final SubtreeSpecification  subtreeSpecification() throws RecognitionException, TokenStreamException {
		SubtreeSpecification ss;
		
		
		log.debug( "entered subtreeSpecification()" );
		// clear out ss, ssModifier, chopBeforeExclusions and chopAfterExclusions
		// in case something is left from the last parse
		ss = null;
		ssModifier = new SubtreeSpecificationModifier();
		chopBeforeExclusions = new HashSet();
		chopAfterExclusions = new HashSet();
		subtreeSpecificationComponentsMonitor = new OptionalComponentsMonitor( 
		new String [] { "base", "specificExclusions", "minimum", "maximum" } );
		
		
		match(OPEN_CURLY);
		{
		_loop528:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop528;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_base:
		case ID_specificExclusions:
		case ID_minimum:
		case ID_maximum:
		{
			subtreeSpecificationComponent();
			{
			_loop531:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop531;
				}
				
			} while (true);
			}
			{
			_loop537:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop534:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop534;
						}
						
					} while (true);
					}
					subtreeSpecificationComponent();
					{
					_loop536:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop536;
						}
						
					} while (true);
					}
				}
				else {
					break _loop537;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
		
		ss = ssModifier.getSubtreeSpecification();
		
		return ss;
	}
	
	public final  UserPermission  userPermission() throws RecognitionException, TokenStreamException {
		 UserPermission l_userPermission ;
		
		
		log.debug( "entered userPermission()" );
		l_userPermission = null;
		userPermissionComponentsMonitor = new MandatoryAndOptionalComponentsMonitor( 
		new String [] { "protectedItems", "grantsAndDenials" }, new String [] { "precedence" } );
		
		
		match(OPEN_CURLY);
		{
		_loop516:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop516;
			}
			
		} while (true);
		}
		anyUserPermission();
		{
		_loop518:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop518;
			}
			
		} while (true);
		}
		{
		_loop524:
		do {
			if ((LA(1)==SEP)) {
				match(SEP);
				{
				_loop521:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop521;
					}
					
				} while (true);
				}
				anyUserPermission();
				{
				_loop523:
				do {
					if ((LA(1)==SP)) {
						match(SP);
					}
					else {
						break _loop523;
					}
					
				} while (true);
				}
			}
			else {
				break _loop524;
			}
			
		} while (true);
		}
		match(CLOSE_CURLY);
		
		if ( !userPermissionComponentsMonitor.finalStateValid() )
		{
		throw new RecognitionException( "Missing mandatory userPermission components: " 
		+ userPermissionComponentsMonitor.getRemainingComponents() );
		}
		
		l_userPermission = new UserPermission( m_aciPrecedence, m_grantsAndDenials, m_protectedItems );
		
		return l_userPermission ;
	}
	
	public final void anyUserPermission() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ID_precedence:
			{
				precedence();
				
				userPermissionComponentsMonitor.useComponent( "precedence" );
				
				break;
			}
			case ID_protectedItems:
			{
				protectedItems();
				
				userPermissionComponentsMonitor.useComponent( "protectedItems" );
				
				break;
			}
			case ID_grantsAndDenials:
			{
				grantsAndDenials();
				
				userPermissionComponentsMonitor.useComponent( "grantsAndDenials" );
				
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( e.getMessage() );
			
		}
	}
	
	public final void subtreeSpecificationComponent() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered subtreeSpecification()" );
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ID_base:
			{
				ss_base();
				
				subtreeSpecificationComponentsMonitor.useComponent( "base" );
				
				break;
			}
			case ID_specificExclusions:
			{
				ss_specificExclusions();
				
				subtreeSpecificationComponentsMonitor.useComponent( "specificExclusions" );
				
				break;
			}
			case ID_minimum:
			{
				ss_minimum();
				
				subtreeSpecificationComponentsMonitor.useComponent( "minimum" );
				
				break;
			}
			case ID_maximum:
			{
				ss_maximum();
				
				subtreeSpecificationComponentsMonitor.useComponent( "maximum" );
				
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (IllegalArgumentException e) {
			
			throw new RecognitionException( e.getMessage() );
			
		}
	}
	
	public final void ss_base() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered ss_base()" );
		LdapDN base = null;
		
		
		match(ID_base);
		{
		int _cnt541=0;
		_loop541:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt541>=1 ) { break _loop541; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt541++;
		} while (true);
		}
		base=distinguishedName();
		
		ssModifier.setBase( base );
		
	}
	
	public final void ss_specificExclusions() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered ss_specificExclusions()" );
		
		
		match(ID_specificExclusions);
		{
		int _cnt544=0;
		_loop544:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt544>=1 ) { break _loop544; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt544++;
		} while (true);
		}
		specificExclusions();
		
		ssModifier.setChopBeforeExclusions( chopBeforeExclusions );
		ssModifier.setChopAfterExclusions( chopAfterExclusions );
		
	}
	
	public final void ss_minimum() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered ss_minimum()" );
		int minimum = 0;
		
		
		match(ID_minimum);
		{
		int _cnt570=0;
		_loop570:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt570>=1 ) { break _loop570; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt570++;
		} while (true);
		}
		minimum=baseDistance();
		
		ssModifier.setMinBaseDistance( minimum );
		
	}
	
	public final void ss_maximum() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered ss_maximum()" );
		int maximum = 0;
		
		
		match(ID_maximum);
		{
		int _cnt573=0;
		_loop573:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				if ( _cnt573>=1 ) { break _loop573; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt573++;
		} while (true);
		}
		maximum=baseDistance();
		
		ssModifier.setMaxBaseDistance( maximum );
		
	}
	
	public final void specificExclusions() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered specificExclusions()" );
		
		
		match(OPEN_CURLY);
		{
		_loop547:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop547;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_chopBefore:
		case ID_chopAfter:
		{
			specificExclusion();
			{
			_loop550:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop550;
				}
				
			} while (true);
			}
			{
			_loop556:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop553:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop553;
						}
						
					} while (true);
					}
					specificExclusion();
					{
					_loop555:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop555;
						}
						
					} while (true);
					}
				}
				else {
					break _loop556;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
	}
	
	public final void specificExclusion() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered specificExclusion()" );
		
		
		switch ( LA(1)) {
		case ID_chopBefore:
		{
			chopBefore();
			break;
		}
		case ID_chopAfter:
		{
			chopAfter();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void chopBefore() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered chopBefore()" );
		LdapDN chopBeforeExclusion = null;
		
		
		match(ID_chopBefore);
		{
		_loop560:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop560;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop562:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop562;
			}
			
		} while (true);
		}
		chopBeforeExclusion=distinguishedName();
		
		chopBeforeExclusions.add( chopBeforeExclusion );
		
	}
	
	public final void chopAfter() throws RecognitionException, TokenStreamException {
		
		
		log.debug( "entered chopAfter()" );
		LdapDN chopAfterExclusion = null;
		
		
		match(ID_chopAfter);
		{
		_loop565:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop565;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop567:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop567;
			}
			
		} while (true);
		}
		chopAfterExclusion=distinguishedName();
		
		chopAfterExclusions.add( chopAfterExclusion );
		
	}
	
	public final  int  baseDistance() throws RecognitionException, TokenStreamException {
		 int distance ;
		
		Token  token = null;
		
		log.debug( "entered baseDistance()" );
		distance = 0;
		
		
		token = LT(1);
		match(INTEGER);
		
		distance = token2Integer( token );
		
		return distance ;
	}
	
	public final  LeafNode  item() throws RecognitionException, TokenStreamException {
		 LeafNode node ;
		
		
		log.debug( "entered item()" );
		node = null;
		String l_oid = null;
		
		
		match(ID_item);
		{
		_loop581:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop581;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop583:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop583;
			}
			
		} while (true);
		}
		l_oid=oid();
		
		node = new SimpleNode( "objectClass" , l_oid , AbstractExprNode.EQUALITY );
		
		return node ;
	}
	
	public final  BranchNode  and() throws RecognitionException, TokenStreamException {
		 BranchNode node ;
		
		
		log.debug( "entered and()" );
		node = null;
		ArrayList children = null; 
		
		
		match(ID_and);
		{
		_loop586:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop586;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop588:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop588;
			}
			
		} while (true);
		}
		children=refinements();
		
		node = new BranchNode( AbstractExprNode.AND , children );
		
		return node ;
	}
	
	public final  BranchNode  or() throws RecognitionException, TokenStreamException {
		 BranchNode node ;
		
		
		log.debug( "entered or()" );
		node = null;
		ArrayList children = null; 
		
		
		match(ID_or);
		{
		_loop591:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop591;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop593:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop593;
			}
			
		} while (true);
		}
		children=refinements();
		
		node = new BranchNode( AbstractExprNode.OR , children );
		
		return node ;
	}
	
	public final  BranchNode  not() throws RecognitionException, TokenStreamException {
		 BranchNode node ;
		
		
		log.debug( "entered not()" );
		node = null;
		ArrayList children = null;
		
		
		match(ID_not);
		{
		_loop596:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop596;
			}
			
		} while (true);
		}
		match(COLON);
		{
		_loop598:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop598;
			}
			
		} while (true);
		}
		children=refinements();
		
		node = new BranchNode( AbstractExprNode.NOT , children );
		
		return node ;
	}
	
	public final  ArrayList  refinements() throws RecognitionException, TokenStreamException {
		 ArrayList children ;
		
		
		log.debug( "entered refinements()" );
		children = null;
		ExprNode child = null;
		ArrayList tempChildren = new ArrayList();
		
		
		match(OPEN_CURLY);
		{
		_loop601:
		do {
			if ((LA(1)==SP)) {
				match(SP);
			}
			else {
				break _loop601;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case ID_item:
		case ID_and:
		case ID_or:
		case ID_not:
		{
			child=refinement();
			{
			_loop604:
			do {
				if ((LA(1)==SP)) {
					match(SP);
				}
				else {
					break _loop604;
				}
				
			} while (true);
			}
			
			tempChildren.add( child );
			
			{
			_loop610:
			do {
				if ((LA(1)==SEP)) {
					match(SEP);
					{
					_loop607:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop607;
						}
						
					} while (true);
					}
					child=refinement();
					{
					_loop609:
					do {
						if ((LA(1)==SP)) {
							match(SP);
						}
						else {
							break _loop609;
						}
						
					} while (true);
					}
					
					tempChildren.add( child );
					
				}
				else {
					break _loop610;
				}
				
			} while (true);
			}
			break;
		}
		case CLOSE_CURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(CLOSE_CURLY);
		
		children = tempChildren;
		
		return children ;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"ATTRIBUTE_VALUE_CANDIDATE",
		"RANGE_OF_VALUES_CANDIDATE",
		"SP",
		"OPEN_CURLY",
		"SEP",
		"CLOSE_CURLY",
		"\"identificationTag\"",
		"SAFEUTF8STRING",
		"\"precedence\"",
		"INTEGER",
		"\"authenticationLevel\"",
		"\"none\"",
		"\"simple\"",
		"\"strong\"",
		"\"itemOrUserFirst\"",
		"\"itemFirst\"",
		"COLON",
		"\"userFirst\"",
		"\"protectedItems\"",
		"\"entry\"",
		"\"allUserAttributeTypes\"",
		"\"attributeType\"",
		"\"allAttributeValues\"",
		"\"allUserAttributeTypesAndValues\"",
		"\"selfValue\"",
		"\"maxValueCount\"",
		"\"type\"",
		"\"maxCount\"",
		"\"maxImmSub\"",
		"\"restrictedBy\"",
		"\"valuesIn\"",
		"\"classes\"",
		"\"itemPermissions\"",
		"\"grantsAndDenials\"",
		"\"grantAdd\"",
		"\"denyAdd\"",
		"\"grantDiscloseOnError\"",
		"\"denyDiscloseOnError\"",
		"\"grantRead\"",
		"\"denyRead\"",
		"\"grantRemove\"",
		"\"denyRemove\"",
		"\"grantBrowse\"",
		"\"denyBrowse\"",
		"\"grantExport\"",
		"\"denyExport\"",
		"\"grantImport\"",
		"\"denyImport\"",
		"\"grantModify\"",
		"\"denyModify\"",
		"\"grantRename\"",
		"\"denyRename\"",
		"\"grantReturnDN\"",
		"\"denyReturnDN\"",
		"\"grantCompare\"",
		"\"denyCompare\"",
		"\"grantFilterMatch\"",
		"\"denyFilterMatch\"",
		"\"grantInvoke\"",
		"\"denyInvoke\"",
		"\"userClasses\"",
		"\"allUsers\"",
		"\"thisEntry\"",
		"\"name\"",
		"\"userGroup\"",
		"\"subtree\"",
		"\"userPermissions\"",
		"\"base\"",
		"\"specificExclusions\"",
		"\"chopBefore\"",
		"\"chopAfter\"",
		"\"minimum\"",
		"\"maximum\"",
		"DESCR",
		"NUMERICOID",
		"\"item\"",
		"\"and\"",
		"\"or\"",
		"\"not\"",
		"\"FALSE\"",
		"\"TRUE\"",
		"\"level\"",
		"\"basicLevels\"",
		"\"localQualifier\"",
		"\"signed\"",
		"\"rangeOfValues\"",
		"\"specificationFilter\"",
		"SAFEUTF8CHAR",
		"DIGIT",
		"LDIGIT",
		"ALPHA",
		"HYPHEN",
		"DOT",
		"INTEGER_OR_NUMERICOID",
		"FILTER",
		"FILTER_VALUE"
	};
	
	
	}
